import * as sanitizeHtml from 'sanitize-html';
import {h} from 'preact';
import {OnClickEvent} from '@playkit-js/common/dist/hoc/a11y-wrapper';
import {ui} from '@playkit-js/kaltura-player-js';
import {UpperBarManager, SidePanelsManager} from '@playkit-js/ui-managers';
import {ObjectUtils, downloadContent, printContent, decodeString} from './utils';
import {icons} from './components/icons';
import {PluginButton} from './components/plugin-button/plugin-button';
import {LiveTranscript} from './components/live-transcript';
import {getConfigValue, isBoolean, makePlainText, prepareCuePoint} from './utils';
import {LiveTranscriptConfig, PluginStates, HighlightedMap, CuePointData, ItemTypes, CuePoint} from './types';

const {SidePanelModes, SidePanelPositions, ReservedPresetNames, ReservedPresetAreas} = ui;
const {withText, Text} = KalturaPlayer.ui.preacti18n;
const {get} = ObjectUtils;

const LOADING_TIMEOUT = 10000;

interface TimedMetadataEvent {
  payload: {
    cues: Array<CuePoint>;
  };
}

export class LiveTranscriptPlugin extends KalturaPlayer.core.BasePlugin {
  static defaultConfig: LiveTranscriptConfig = {
    expandMode: SidePanelModes.ALONGSIDE,
    expandOnFirstPlay: true,
    showTime: true,
    position: SidePanelPositions.RIGHT,
    scrollOffset: 0,
    searchDebounceTimeout: 250,
    searchNextPrevDebounceTimeout: 100,
    downloadDisabled: false,
    printDisabled: false
  };
  private _activeCaptionMapId: string = '';
  private _activeCuePointsMap: HighlightedMap = {};
  private _activeTextTrack: any = null;
  private _captionMap: Map<string, Array<CuePointData>> = new Map();
  private _isLoading = false;
  private _hasError = false;
  private _triggeredByKeyboard = false;

  private _liveTranscriptPanel = -1;
  private _liveTranscriptIcon = -1;
  private _pluginState: PluginStates | null = null;
  private _pluginButtonRef: HTMLButtonElement | null = null;
  private _data: CuePointData[] = [];

  constructor(name: string, player: KalturaPlayerTypes.Player, config: LiveTranscriptConfig) {
    super(name, player, config);
  }

  get sidePanelsManager() {
    return this.player.getService('sidePanelsManager') as SidePanelsManager | undefined;
  }

  get upperBarManager() {
    return this.player.getService('upperBarManager') as UpperBarManager | undefined;
  }

  get cuePointManager() {
    return this.player.getService('kalturaCuepoints') as any;
  }

  loadMedia(): void {
    this.player.ready().then(()=> {
      if (!this.sidePanelsManager || !this.upperBarManager) {
        this.logger.warn("sidePanelsManager or upperBarManager haven't registered");
        return;
      }

      this._initListeners();

      this.eventManager.listenOnce(
          this.player,
          this.player.Event.Core.FIRST_PLAYING,
          () => {
            if (!this.player.isLive()) {
              this.logger.warn("Live transcript plugin is not supported for non-live entries");
              return;
            }

            this._addTranscriptItem();
          },
      );
    });
  }

  private _initListeners(): void {
    this.eventManager.listenOnce(this.player, this.player.Event.TRACKS_CHANGED, () => {
      if (this._getTextTracks().length) {
        this.eventManager.listen(this.player, this.player.Event.TIMED_METADATA_CHANGE, this._onTimedMetadataChange);
        this.eventManager.listen(this.player, this.player.Event.TIMED_METADATA_ADDED, this._onTimedMetadataAdded);
        this.eventManager.listen(this.player, this.player.Event.TEXT_TRACK_CHANGED, this._handleLanguageChange);

        const allTextTracks = this._getTextTracks();
        const activePlayerTextTrack = allTextTracks.find((track: any) => track.active);

        const TextTracks = Object.values(this.player.getVideoElement().textTracks);
        if (activePlayerTextTrack.language === "off") {
          this._activeTextTrack = TextTracks.find((track: any) => {
            return track.kind === "captions";
          });
        } else {
          this._activeTextTrack = TextTracks.find((track: any) => {
            return track.language === activePlayerTextTrack.language && track.kind === "captions";
          });
        }

        this._activeTextTrack.addEventListener('cuechange', (e: Event) => {
          const target = e.currentTarget as TextTrack;
          target.mode = "hidden"; // this override allows for live transcript to work even if Closed Captions are turned off
          if (this._data.length === 0) {
            const pastCues = target.cues as TextTrackCueList;
            this._data = Object.values(pastCues).map((cue: any) => ({
              id: (KalturaPlayer.core.utils as any).Generator.guid(), // we are generating unique ids here because otherwise some cues have the same id (maybe in the test video only, not sure)
              startTime: cue.startTime,
              text: cue.text
            }));
          }

          const activeCues = target.activeCues as TextTrackCueList;
          const additionalData: CuePointData[] = Object.values(activeCues).map((cue: any) => ({
            id:  (KalturaPlayer.core.utils as any).Generator.guid(), // we are generating unique ids here because otherwise some cues have the same id (maybe in the test video only, not sure)
            startTime: cue.startTime,
            text: cue.text
          }));
          this._data = [...this._data, ...additionalData.reverse()];

          this._updateLiveTranscriptPanel();
        });
      }
    });
  }

  private _initLoading = () => {
    this._isLoading = false;
    this._hasError = false;
    this._updateLiveTranscriptPanel();
  };

  private _handleLanguageChange = () => {
    this._activeCaptionMapId = this._getCaptionMapId();
    this._initLoading();
  };

  private _updateLiveTranscriptPanel() {
    if (this._liveTranscriptPanel) {
      this.sidePanelsManager?.update(this._liveTranscriptPanel);
    }
  }

  private _onTimedMetadataAdded = ({payload}: TimedMetadataEvent) => {
    const captionData: CuePointData[] = [];
    payload.cues.forEach((cue: CuePoint) => {
      if (cue.metadata.cuePointType === ItemTypes.Caption) {
        captionData.push(prepareCuePoint(cue));
      }
    });
    if (captionData.length) {
      this._addCaptionData(captionData);
      this._addTranscriptItem();
    }
  };

  private _onTimedMetadataChange = ({payload}: TimedMetadataEvent) => {
    const transcriptCuePoints: Array<CuePoint> = payload.cues
      .filter((cue: CuePoint) => {
        return cue.metadata.cuePointType === ItemTypes.Caption;
      })
      .filter((cue, index, array) => {
        // filter out captions that has endTime eq to next caption startTime
        const nextCue = array[index + 1];
        return !nextCue || cue.endTime !== nextCue.startTime;
      });
    this._activeCuePointsMap = {};
    transcriptCuePoints.forEach(cue => {
      this._activeCuePointsMap[cue.id] = true;
    });
    this._updateLiveTranscriptPanel();
  };

  private _addCaptionData = (newData: CuePointData[]) => {
    this._activeCaptionMapId = this._getCaptionMapId();
    this._captionMap.set(this._activeCaptionMapId, this._sanitizeCaptions(newData));
    this._isLoading = false;
    this._updateLiveTranscriptPanel();
  };

  private _getTextTracks = () => {
    return this.player.getTracks(this.player.Track.TEXT) || [];
  };

  private _getCaptionMapId = (): string => {
    const allTextTracks = this._getTextTracks();
    const activeTextTrack = allTextTracks.find(track => track.active);
    if (activeTextTrack?.language === 'off') {
      // use 1st captions from text-track list
      return `${allTextTracks[0]?.language}-${allTextTracks[0]?.label}`;
    }
    return `${activeTextTrack?.language}-${activeTextTrack?.label}`;
  };

  private _activatePlugin = () => {
    this.ready.then(() => {
      this.sidePanelsManager?.activateItem(this._liveTranscriptPanel);
      this._pluginState = PluginStates.OPENED;
      this.upperBarManager?.update(this._liveTranscriptIcon);
    });
  };

  private _deactivatePlugin = () => {
    this.ready.then(() => {
      this.sidePanelsManager?.deactivateItem(this._liveTranscriptPanel);
      this.upperBarManager?.update(this._liveTranscriptIcon);
    });
  };

  private _isPluginActive = () => {
    return this.sidePanelsManager!.isItemActive(this._liveTranscriptPanel);
  };

  private _handleClickOnPluginIcon = (e: OnClickEvent, byKeyboard?: boolean) => {
    if (this._isPluginActive()) {
      this._triggeredByKeyboard = false;
      this._deactivatePlugin();
      this._pluginState = PluginStates.CLOSED;
    } else {
      this._triggeredByKeyboard = Boolean(byKeyboard);
      this._activatePlugin();
    }
  };

  private _sanitizeCaptions = (data: CuePointData[]) => {
    return data.map(caption => ({
      ...caption,
      text: decodeString(
        sanitizeHtml(caption.text || '', {
          allowedAttributes: {},
          allowedTags: []
        })
      )
    }));
  };

  private _addTranscriptItem(): void {
    if (Math.max(this._liveTranscriptPanel, this._liveTranscriptIcon) > 0) {
      // transcript panel or icon already exist
      return;
    }

    const {
      expandMode,
      position,
      expandOnFirstPlay,
      showTime,
      scrollOffset,
      searchDebounceTimeout,
      searchNextPrevDebounceTimeout,
    } = this.config;
    this._liveTranscriptPanel = this.sidePanelsManager!.add({
      label: 'Live Transcript',
      panelComponent: () => {
        return (
          <LiveTranscript
            showTime={getConfigValue(showTime, isBoolean, true)}
            scrollOffset={getConfigValue(scrollOffset, Number.isInteger, 0)}
            searchDebounceTimeout={getConfigValue(searchDebounceTimeout, Number.isInteger, 250)}
            searchNextPrevDebounceTimeout={getConfigValue(searchNextPrevDebounceTimeout, Number.isInteger, 100)}
            highlightedMap={this._activeCuePointsMap}
            onSeek={this._seekTo}
            onItemClicked={this._seekTo}
            expandMode={this.config.expandMode}
            captions={this._data}
            isLoading={false}
            hasError={this._hasError}
            onRetryLoad={this._updateLiveTranscriptPanel}
            currentTime={this.player.currentTime}
            videoDuration={this.player.duration}
            kitchenSinkActive={this._isPluginActive()}
            toggledWithEnter={this._triggeredByKeyboard}
            onClose={this._handleClose}
            downloadDisabled={true}
            onDownload={this._handleDownload}
            printDisabled={true}
            onPrint={this._handlePrint}
          />
        );
      },
      presets: [ReservedPresetNames.Playback, ReservedPresetNames.Live, ReservedPresetNames.Ads],
      position: position,
      expandMode: expandMode === SidePanelModes.ALONGSIDE ? SidePanelModes.ALONGSIDE : SidePanelModes.OVER,
      onDeactivate: this._deactivatePlugin
    }) as number;
    const translates = {
      showTranscript: <Text id="transcript.show_plugin">Show Live Transcript</Text>,
      hideTranscript: <Text id="transcript.hide_plugin">Hide Live Transcript</Text>
    };
    this._liveTranscriptIcon = this.upperBarManager!.add({
      label: 'Live Transcript',
      svgIcon: {path: icons.PLUGIN_ICON, viewBox: `0 0 ${icons.BigSize} ${icons.BigSize}`},
      onClick: this._handleClickOnPluginIcon as () => void,
      component: withText(translates)((props: {showTranscript: string; hideTranscript: string}) => {
        const isActive = this._isPluginActive();
        const label = isActive ? props.hideTranscript : props.showTranscript;
        return (
          <PluginButton
            isActive={isActive}
            id="liveTranscript-icon"
            label={label}
            icon={icons.PLUGIN_ICON}
            dataTestId="liveTranscript_pluginButton"
            setRef={this._setPluginButtonRef}
          />
        );
      })
    }) as number;

    if ((expandOnFirstPlay && !this._pluginState) || this._pluginState === PluginStates.OPENED) {
      this._activatePlugin();
    }
  }

  private _setPluginButtonRef = (ref: HTMLButtonElement | null) => {
    this._pluginButtonRef = ref;
  };

  private _seekTo = (time: number) => {
    this.player.currentTime = time;
  };

  private _handleDownload = () => {
    //
  };

  private _handlePrint = () => {
    //
  };

  private _handleClose = (e: OnClickEvent, byKeyboard: boolean) => {
    if (byKeyboard) {
      this._pluginButtonRef?.focus();
    }
    this._deactivatePlugin();
    this._pluginState = PluginStates.CLOSED;
  };

  static isValid(): boolean {
    return true;
  }

  reset(): void {
    this.eventManager.removeAll();
    if (Math.max(this._liveTranscriptPanel, this._liveTranscriptIcon) > 0) {
      this.sidePanelsManager?.remove(this._liveTranscriptPanel);
      this.upperBarManager!.remove(this._liveTranscriptIcon);
      this._liveTranscriptPanel = -1;
      this._liveTranscriptIcon = -1;
      this._pluginButtonRef = null;
    }
    this._captionMap = new Map();
    this._activeCaptionMapId = '';
    this._isLoading = false;
    this._hasError = false;
    this._triggeredByKeyboard = false;
  }

  destroy(): void {
    this.reset();
  }
}
